/******************************************************************************
*                                                                             *
*  Modeless dialog box example. This demonstrates how to allow an unlimited   *
*     number of modeless dialog boxes without having to track each for proper *
*     dialog box message handling of the Tab key and so on. The idea is to    *
*     have each modeless dialog box maintain a global "currently active"      *
*     variable which is then tested in the message loop, providing "normal"   *
*     dialog box operation. Easy, but not intuitive.                          *
*                                                                             *
******************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include "windows.h"

// == Function Prototypes =====================================================

LRESULT WINAPI MainWndProc( HWND, UINT, WPARAM, LPARAM );
BOOL WINAPI ModelessDlgProc( HWND, UINT, WPARAM, LPARAM );

// == Global Variables ========================================================

HINSTANCE hInst;                                //  Application instance handle
HWND hMainWnd;                                           //  Main window handle
HWND hModelessDlg;       //  Global to hold the "currently selected" dialog box

// == Application Entry Point =================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
                      int nShow )
{
   WNDCLASS wc;
   MSG msg;

   wc.style = 0;
   wc.hInstance = hInstance;
   wc.lpszClassName = "MODELESS";
   wc.lpszMenuName = "MODELESS";
   wc.lpfnWndProc = MainWndProc;
   wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
   wc.hCursor = LoadCursor( NULL, IDC_ARROW );
   wc.hbrBackground = (HBRUSH)( COLOR_WINDOW + 1 );
   wc.cbClsExtra = wc.cbWndExtra = 0;        //  Register the main window class
   if( ! RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance;                        //  Save instance handle in global

   hMainWnd = CreateWindow( "MODELESS", "Modeless Dialogs",  //  And create the
                            WS_OVERLAPPEDWINDOW,                //  main window
                            CW_USEDEFAULT, CW_USEDEFAULT,
                            CW_USEDEFAULT, CW_USEDEFAULT,
                            NULL, NULL, hInst, NULL );
   if( ! hMainWnd )
      return( FALSE );                            //  If the window failed, die

   hModelessDlg = NULL;   //  Initialize "current" dialog handle to "no dialog"

   ShowWindow( hMainWnd, nShow );        //  Show the main window and we're off
   UpdateWindow( hMainWnd );

   while( GetMessage( &msg, NULL, 0, 0 ) )         //  Usual message loop stuff
   {
      if( hModelessDlg )    //  If there is a "current dialog" and this message
         if( IsDialogMessage( hModelessDlg, &msg ) )     //  is for it, process
            continue;                  //  and go for next message in the queue

      TranslateMessage( &msg );       //  Otherwise, do the usual message stuff
      DispatchMessage( &msg );
   }
   return( FALSE );                                                //  All done
}

// == Main Window Procedure ===================================================

LRESULT WINAPI MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_DESTROY:
         PostQuitMessage( 0 );          // Kill the app if the window is killed
         break;                       //  Lots of killing around here, ya know?

      case WM_COMMAND:
         if( wParam == 100 )                //  More killing...kill this window
         {                                     //  if the menu item is selected
            DestroyWindow( hWnd );    //  which then, you got it, kills the app
            break;                   //  Yeah...die....die!....heh hem heh.....
         }
         if( wParam == 101 )               //  Create a new modeless dialog box
         {                     //  Nothing's critical, so the return is ignored
            CreateDialog( hInst, MAKEINTRESOURCE( 10000 ), hWnd,
                          ModelessDlgProc );
            break;
         }
         break;

      default:                                 //  Usual window procedure stuff
         return( DefWindowProc( hWnd, msg, wParam, lParam ) );
   }
   return( FALSE );
}                                                      //  End of MainWndProc()

// == Modeless Dialog Box Procedure ===========================================

BOOL WINAPI ModelessDlgProc( HWND hDlg, UINT msg, WPARAM wParam,
                             LPARAM lParam )
{
   switch( msg )
   {
      case WM_INITDIALOG:                           //  Normal dialog box stuff
         return( TRUE );

      case WM_ACTIVATE:          //  If the dialog active status has changed...
         if( LOWORD( wParam ) == WA_INACTIVE )              //  Is it inactive?
            hModelessDlg = NULL;      //  If so, set global variable to nothing
         else
            hModelessDlg = hDlg;  //  Otherwise, this is the current dialog box
         break;

      case WM_COMMAND:
         if( wParam == IDCANCEL )          // Kill dialog is Cancel is selected
         {
            DestroyWindow( hDlg );
            return( TRUE );
         }
         break;
   }
   return( FALSE );
}                                                  //  End of ModelessDlgProc()

// ============================================================================
